'''vcf2vcf.py - manipulate vcf files
=================================

Purpose
-------

Manipulate vcf-formatted files.


Usage
-----

Type::

   python vcf2vcf.py --help

for command line usage.

Methods
-------

This script provides the following methods:

re-order
   reorder sample columns in vcf formatted file according to a given sort order

Documentation
-------------

This is a tool for manipulating vcf-formatted files.  The following
options are available:

+-----------+-------------------------+
+-----------+-------------------------+

lift-over
^^^^^^^^^

Command line options
--------------------

'''

import os
import sys
import random
import collections
import numpy
import re
import pysam
import quicksect
import cgatcore.experiment as E
import cgatcore.iotools as iotools


def read_liftover_chain(infile):

    E.debug("started reading mapping information")

    map_id2chromosome = ["", ]
    map_chromosome2id = {}
    n = 0

    Chain = collections.namedtuple(
        "Chain",
        ["score",
         "target_name", "target_size", "target_strand",
         "target_start", "target_end",
         "query_name", "query_size", "query_strand",
         "query_start", "query_end", "chainid"])

    def blocks(infile):

        keep = False
        for line in infile:
            if line.startswith("chain"):
                chain_data = Chain._make(line[:-1].split(" ")[1:])

                if chain_data.target_strand == "-":
                    raise NotImplementedError("target strand is negative")
                alignment_data = []
            elif line.strip() == "":
                yield chain_data, alignment_data
            else:
                alignment_data.append(list(map(int, line.split(("\t")))))

    map_chromosomes = collections.defaultdict(
        quicksect.IntervalTree)
    map_contig2length = collections.defaultdict(int)

    for chain_data, alignment_data in blocks(infile):

        map_contig2length[chain_data.query_name] = int(chain_data.query_size)

        # target maps to query
        # coordinates are zero-based, half-open When
        # the strand value is "-", position coordinates are listed in
        # terms of the reverse-complemented sequence
        x = int(chain_data.target_start)
        y = int(chain_data.query_start)
        # revert coordinates for negative strands (it seems that
        # the mapping file uses reverse coordinates, while liftover
        # output doesn't)
        invert = chain_data.query_strand == "-"
        mm = map_chromosomes[chain_data.target_name]

        for d in alignment_data:
            if len(d) == 3:
                size, increment_x, increment_y = d
            else:
                size, increment_x, increment_y = d[0], 0, 0

            mm.add(x, x + size,
                   (chain_data.query_name,
                    y,
                    y + size,
                    invert))

            x += increment_x + size
            y += increment_y + size

            if y < 0:
                raise ValueError(
                    "illegal mapping in chain {}".format(chain_data))

    return map_chromosomes, map_contig2length


def main(argv=None):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "--input-filename-fasta", dest="input_filename_fasta", type="string",
        help="filename with reference sequence in fasta format [%default]")

    parser.add_option(
        "--input-filename-bam", dest="input_filename_bam", type="string",
        help="filename with aligned reads [%default]")

    parser.add_option(
        "--method", dest="methods", type="choice", action="append",
        choices=["add-strelka-genotype",
                 "lift-over"],
        help="methods to apply [%default]")

    parser.add_option(
        "--input-filename-chain", dest="input_filename_chain", type="string",
        help="filename with alignment chain for lift-over [%default]")

    parser.add_option(
        "--normal-sample-regex", dest="normal_sample_regex", type="string",
        help="regular expression to apply to header to identify normal "
        "sample id [%default]")

    parser.add_option(
        "--output-filename-unmapped", dest="output_filename_unmapped", type="string",
        help="filename with variants that could not be lifted over [%default]")

    parser.set_defaults(
        input_filename_fasta=None,
        input_filename_bam=None,
        input_filename_vcf="-",
        sample_size=0.001,
        region_size=20,
        methods=[],
        normal_sample_regex=None,
        input_filename_chain=None,
        output_filename_unmapped=None,
    )

    (options, args) = E.start(parser,
                              argv=argv,
                              add_output_options=True)

    if len(args) > 0:
        options.input_filename_vcf = args[0]

    vcf_in = pysam.VariantFile(options.input_filename_vcf)

    if "lift-over" in options.methods:
        if options.input_filename_chain is None:
            raise ValueError("--method=lift-over requires --input-filename-chain")
        if not os.path.exists(options.input_filename_chain):
            raise OSError("file {} with chain data does not exist".format(
                options.input_filename_chain))
        E.info("reading chain from {}".format(options.input_filename_chain))
        with iotools.open_file(options.input_filename_chain) as inf:
            map_chain, map_contig2length = read_liftover_chain(inf)

    if options.input_filename_fasta:
        fasta = pysam.FastaFile(options.input_filename_fasta)
    else:
        fasta = None

    if options.input_filename_bam:
        bam = pysam.AlignmentFile(options.input_filename_bam)
    else:
        bam = None

    outf = options.stdout

    c = E.Counter()

    if "add-strelka-genotype" in options.methods:
        map_nt2gt = {"ref": "0/0",
                     "het": "0/1",
                     "hom": "1/1",
                     "conflict": "."}

        map_tumour2gt = {"ref": "0/0",
                         "het": "0/1",
                         "hom": "1/1"}

        header = str(vcf_in.header).splitlines()

        header.insert(
            len(header) - 1,
            '##FORMAT=<ID=GT,Number=1,Type=String,Description='
            '"Genotypes of reference and alternative alleles, '
            'added by cgatcore vcf2vcf.">')

        header = "\n".join(header)
        if options.normal_sample_regex:
            normal_sample = re.search(" -bam-file \S+/([^/]+)_S\d+.bam", header).groups()[0]
        else:
            normal_sample = "NORMAL"

        is_first = True

        for record in vcf_in:
            c.input += 1

            if "GT" in record.format:
                if is_first:
                    outf.write(header + "\n")
                    is_first = False
                outf.write(str(record))
                c.has_gt += 1
                continue

            gt_normal = map_nt2gt[record.info["NT"]]
            gt_tumour = record.info["SGT"]
            norm, tumour = gt_tumour.split("->")
            if gt_tumour[0] in "ACGT":
                alts = record.alts
                if alts is None:
                    c.no_alt += 1
                    continue

                if len(record.alts) > 1:
                    c.multi_allelic += 1
                    continue

                _map_tumour2gt = {
                    record.alts[0]: "1",
                    record.ref: "0"}
                try:
                    gt_tumour = "/".join(
                        sorted([_map_tumour2gt[x] for x in tumour]))
                except KeyError:
                    gt_tumour = "."
                    c.ambigous_genotype += 1
            else:
                gt_tumour = map_tumour2gt[tumour]

            fields = str(record)[:-1].split("\t")
            # FORMAT
            fields[8] = ":".join(("GT", fields[8]))
            # SAMPLES
            # makes a few assumptions, fix!
            header_insert_normal = False
            if len(fields) == 11:
                fields[9] = ":".join((gt_normal, fields[9]))
                fields[10] = ":".join((gt_tumour, fields[10]))
            elif len(fields) == 10:
                header_insert_normal = True
                values = fields[9].split(":")
                fields.append(":".join((gt_tumour, fields[9])))
                fields[9] = ":".join([gt_normal] + ["."] * len(values))
            else:
                raise NotImplementedError()

            if is_first:
                if not header_insert_normal:
                    outf.write(header + "\n")
                else:
                    header = re.sub(r"\tFORMAT\t",
                                    "\tFORMAT\t%s\t" % normal_sample, header)
                    outf.write(header + "\n")
                is_first = False
            outf.write("\t".join(fields) + "\n")
            c.output += 1

    elif "lift-over" in options.methods:
        header = str(vcf_in.header).splitlines()

        if fasta:
            # validate contig size
            expected_lengths = dict(list(zip(fasta.references, fasta.lengths)))
        else:
            expected_lengths = map_contig2length

        # update contig names and sizes in VCF header
        header = [x for x in header if not x.startswith("##contig")]
        header[-1:-1] = ["##contig=<ID={},length={}>".format(
            contig, length) for contig, length in sorted(expected_lengths.items())]

        header.insert(
            len(header) - 1,
            '##liftover=<CHAIN={},REFERENCE={}>'.format(
                options.input_filename_chain,
                options.input_filename_fasta))
        outf.write("\n".join(header) + "\n")

        unmapped_contigs = set()
        unknown_contigs = set()

        trans_genotypes = str.maketrans("01", "10")

        if fasta:
            # validate contig size
            expected_lengths = dict(list(zip(fasta.references, fasta.lengths)))
            for contig, length in list(map_contig2length.items()):
                if contig in expected_lengths:
                    if length != expected_lengths[contig]:
                        raise ValueError(
                            "contig lengths mismatch. For contig {} chain files "
                            "says {}, but fasta files says {}".format(
                                contig, length, expected_lengths[contig]))
            E.info("contig sizes in chain file and fasta files correspond.")

        if options.output_filename_unmapped:
            outfile_unmapped = iotools.open_file(options.output_filename_unmapped, "w")
            outfile_unmapped.write("\n".join(header) + "\n")
        else:
            outfile_unmapped = None

        for record in vcf_in:
            c.input += 1

            try:
                mm = map_chain[record.contig]
            except KeyError:
                c.skipped_unmapped_contig += 1
                unmapped_contigs.add(record.contig)
                if outfile_unmapped:
                    outfile_unmapped.write("skipped_unmapped_contig\t{}".format(str(record)))
                continue

            try:
                m = mm.search(record.start, record.stop)
            except AttributeError:
                c.skipped_mapping_error += 1
                if outfile_unmapped:
                    outfile_unmapped.write("skipped_mapping_error\t{}".format(str(record)))
                continue

            if len(m) == 0:
                c.skipped_unmapped_position += 1
                if outfile_unmapped:
                    outfile_unmapped.write("skipped_unmapped_position\t{}".format(str(record)))
                continue
            elif len(m) > 1:
                c.skipped_multimapping_position += 1
                if outfile_unmapped:
                    outfile_unmapped.write("skipped_multimapping_position\t{}".format(str(record)))
                continue

            m = m[0]
            y_contig, y_start, y_end, y_invert = m.data

            if y_invert:
                y_pos = y_end - (record.start - m.start)
            else:
                y_pos = (record.start - m.start) + y_start

            if fasta:
                try:
                    ref_base = fasta.fetch(y_contig, y_pos, y_pos + len(record.ref)).upper()
                except KeyError:
                    c.skipped_unknown_contig += 1
                    unknown_contigs.add(y_contig)
                    ref_base = None
                    continue

            swap_alleles = False
            if ref_base:
                error = False
                if ref_base == record.ref:
                    c.matches += 1
                else:
                    if len(record.alts) == 1:
                        alt_base = record.alts[0]
                        if ref_base == alt_base:
                            swap_alleles = True
                            c.allele_swap_variant += 1
                        else:
                            c.error_mismatch_variant += 1
                            error = "mismatch"
                    else:
                        error = "multi-mismatch"
                        c.error_multi_mismatch_variant += 1

                if error:
                    if outfile_unmapped:
                        outfile_unmapped.write("{}\t{}".format(error, str(record)))
                    c.skipped_error_variant += 1
                    continue

            fields = str(record)[:-1].split("\t")
            fields[0] = y_contig
            fields[1] = str(y_pos)

            if swap_alleles:
                fields[4] = alt_base
                fields[5] = ref_base
                # update genotype fields
                keep = False
                for idx in range(9, len(fields)):
                    gt, rest = fields[idx].split(":", 1)
                    keep = keep or "0" in gt
                    fields[idx] = ":".join((gt.translate(trans_genotypes), rest))

                # remove reference only calls
                if not keep:
                    if outfile_unmapped:
                        outfile_unmapped.write("reference_call\t{}".format(str(record)))
                    c.skipped_allele_swap_reference += 1
                continue

            c.output += 1
            outf.write("\t".join(fields) + "\n")

        c.unmapped_contigs = len(unmapped_contigs)
        c.unknown_contigs = len(unknown_contigs)

        E.info(c.asTable())
        if unknown_contigs:
            E.info("unknown contigs: {}".format(",".join(sorted(unknown_contigs))))
        if unmapped_contigs:
            E.info("unmapped contigs: {}".format(",".join(sorted(unmapped_contigs))))

    E.stop()


if __name__ == "__main__":
    sys.exit(main())




import sys
import os
import glob
import tempfile
import argparse
import subprocess
import time
from cgatcore import iotools as iotools


def stop_err(msg):
    sys.stderr.write('%s\n' % msg)
    sys.exit()


def timenow():
    """return current time as a string
    """
    return time.strftime('%d/%m/%Y %H:%M:%S', time.localtime(time.time()))


class cgatBase():

    """
    simple base class with some utilities for Picard
    adapted and merged with Kelly Vincent's code april 2011 Ross
    lots of changes...
    """

    def __init__(self, opts=None, arg0=None):
        """ common stuff needed at init for a picard tool
        """

    def baseName(self, name=None):
        return os.path.splitext(os.path.basename(name))[0]

    def setLogging(self, logfname="picard_wrapper.log"):
        """setup a logger
        """
        logging.basicConfig(level=logging.INFO,
                            filename=logfname,
                            filemode='a')

    def readLarge(self, fname=None):
        """ read a potentially huge file.
        """
        try:
            # get stderr, allowing for case where it's very large
            tmp = iotools.open_file(fname, 'rb')
            s = ''
            buffsize = 1048576
            try:
                while True:
                    more = tmp.read(buffsize)
                    if len(more) > 0:
                        s += more
                    else:
                        break
            except OverflowError:
                pass
            tmp.close()
        except Exception as e:
            stop_err('Read Large Exception : %s' % str(e))
        return s

    def runStatement(self, cl=None, output_dir=None):
        """ construct and run a command line
        we have galaxy's temp path as opt.temp_dir so don't really need isolation
        sometimes stdout is needed as the output - ugly hacks to deal with potentially vast artifacts
        """
        assert cl is not None, 'PicardBase runCL needs a command line as cl'
        process = subprocess.Popen(cl, shell=True)
        rval = process.wait()

    def runPic(self, jar, cl):
        """
        cl should be everything after the jar file name in the command
        """
        runme = ['java -Xmx%s' % self.opts.maxjheap]
        runme.append(" -Djava.io.tmpdir='%s' " % self.opts.tmpdir)
        runme.append('-jar %s' % jar)
        runme += cl
        s, stdouts, rval = self.runCL(cl=runme, output_dir=self.opts.outdir)
        return stdouts, rval

    def samToBam(self, infile=None, outdir=None):
        """
        use samtools view to convert sam to bam
        """
        fd, tempbam = tempfile.mkstemp(dir=outdir, suffix='rgutilsTemp.bam')
        cl = ['samtools view -h -b -S -o ', tempbam, infile]
        tlog, stdouts, rval = self.runCL(cl, outdir)
        return tlog, tempbam, rval

    def sortSam(self, infile=None, outfile=None, outdir=None):
        """
        """
        print('## sortSam got infile=%s,outfile=%s,outdir=%s' % (infile, outfile, outdir))
        cl = ['samtools sort', infile, outfile]
        tlog, stdouts, rval = self.runCL(cl, outdir)
        return tlog

    def cleanup(self):
        for fname in self.delme:
            try:
                os.unlink(fname)
            except:
                pass


def __main__():

    # use argparse to ignore unknown options
    parser = argparse.ArgumentParser()

    parser.add_argument("--version", action="version", version="%(prog)s")
    parser.add_argument("--wrapper-command", dest="command", type=str)
    parser.add_argument("--wrapper-bam-file", dest="bam_file", type=str)
    parser.add_argument("--wrapper-bam-option", dest="bam_option", type=str)
    parser.add_argument("--wrapper-bai-file", dest="bai_file", type=str)
    parser.add_argument(
        "--wrapper-dry-run", dest="dry_run", action="store_true")
    parser.add_argument("--wrapper-html-dir", dest="html_dir", type=str)
    parser.add_argument("--wrapper-html-file", dest="html_file", type=str)

    options, unknown = parser.parse_known_args()

    cgat = cgatBase(options)

    option_map = []

    if options.bai_file or options.bam_file:
        if not (options.bai_file and options.bam_file):
            raise ValueError(
                "wrapper called with bam or bai file, but not both")

        if not options.bam_option:
            options.bam_option = "bam-file"

        tmp_fd, tmp_name = tempfile.mkstemp()
        tmp_bam_name = '%s.bam' % tmp_name
        tmp_bai_name = '%s.bai' % tmp_bam_name
        os.symlink(options.bam_file, tmp_bam_name)
        os.symlink(options.bai_file, tmp_bai_name)
        if options.bam_option.startswith("--"):
            # long option
            option_map.append("%s=%s" % (options.bam_option, tmp_bam_name))
        else:
            # short option
            option_map.append("%s %s" % (options.bam_option, tmp_bam_name))

    if options.html_dir:
        os.mkdir(options.html_dir)
        option_map.append("%s=%s/%%s" %
                          ("--output-filename-pattern", options.html_dir))

    statement = "python " + " ".join([options.command] + unknown + option_map)

    if options.dry_run:
        sys.stdout.write(statement + "\n")
        return

    else:
        cgat.runStatement(statement)

    if options.bai_file:
        os.unlink(tmp_bam_name)
        os.unlink(tmp_bai_name)

    if options.html_file:
        with iotools.open_file(options.html_file, "w") as outf:
            outf.write('<h1>%s - Output</h1>' %
                       os.path.basename(options.wrapper_command))
            for fn in glob.glob(os.path.join(options.html_dir, "*.*")):
                dirname, basename = os.path.split(fn)
                outf.write('''<li><a href="%s">%s</a></li>\n''' %
                           (basename, basename))


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv


if __name__ == "__main__":
    sys.exit(main(sys.argv))



"""bam2libtype.py - determine the library type of a bam file
============================================================

Author: Adam Cribbs

Purpose
-------

This tool determines the library type of a BAM file. The naming
convention used is from the salmon documentation:
http://salmon.readthedocs.io/en/latest/library_type.html.

BAM files need to have a corresponding index file i.e. example.bam
and example.bam.bai

For single-end data

    Determining which read the strand is on is straightforward using pysam
    function .is_reversed.

For paired-end data

    The relative position of read1 and read2 needs to be determined including
    orientation relative to each other.


Usage
-----

    cat example.bam | cgat bam2libtype > out.tsv

options
-------

There are no options for this script, just pass the script a bam file
as the stdin and an outfile as the stdout.


Type::

   python bam2bed.py --help

for command line help.

Command line options
--------------------
"""

import sys
import pysam
import cgatcore.experiment as E


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(
        version="%prog version: $Id$", usage=globals()["__doc__"])

    parser.add_option(
        "-i", "--max-iterations", type="int",
        help="maximum number of iterations. Set to 0 to go through all reads "
        "[%default]")

    parser.set_defaults(
        max_iteratiors=10000
    )

    (options, args) = E.start(parser, argv=argv)

    samfile = pysam.AlignmentFile(options.stdin, "rb")
    outfile = options.stdout

    # initialise counts for each library type
    MSR = 0
    MSF = 0
    ISF = 0
    ISR = 0
    OSF = 0
    OSR = 0
    SR = 0
    SF = 0

    reads_processed = set()

    for iteration, read in enumerate(samfile):

        if options.max_iterations and iteration > int(options.max_iterations):
            break

        if read.qname not in reads_processed:
            reads_processed.add(read.qname)
        else:
            continue

        # to handle paired end reads:
        if read.is_paired and read.is_proper_pair:
            # get attributes of read
            read_start = read.reference_start
            read_end = read.reference_end
            read_neg = read.is_reverse

            # specify which read is R1 and which is R2:
            # specify which read is R1 and which is R2:
            if read.is_read1 is True:
                R1_is_reverse = read.is_reverse
                R1_reference_start = read.reference_start

                R2_is_reverse = read.mate_is_reverse
                R2_reference_start = read.next_reference_start
            else:
                R1_is_reverse = read.mate_is_reverse
                R1_reference_start = read.next_reference_start

                R2_is_reverse = read.is_reverse
                R2_reference_start = read.reference_start

                # Decision tree to specify strandness:
                # potential to convert this to a machine learning
                # decision tree algorithm in the future:
            if R1_is_reverse is True:

                if R2_is_reverse is True:

                    MSF += 1
                else:
                    if R2_reference_start - R1_reference_start >= 0:
                        OSR += 1
                    else:
                        ISR += 1

            else:

                if R2_is_reverse is True:

                    if R1_reference_start - R2_reference_start >= 0:

                        OSF += 1
                    else:
                        ISF += 1
                else:
                    MSR += 1
        else:
            if read.is_reverse:
                SR += 1
            else:
                SF += 1

    total = MSR + ISR + OSR + ISF + MSF + OSF + SF + SR

    def total_percent(strand, total):
        return float(strand)/float(total)*100

    MSR_total = total_percent(MSR, total)
    ISR_total = total_percent(ISR, total)
    OSR_total = total_percent(OSR, total)
    ISF_total = total_percent(ISF, total)
    MSF_total = total_percent(MSF, total)
    OSF_total = total_percent(OSF, total)
    SF_total = total_percent(SF, total)
    SR_total = total_percent(SR, total)

    outfile.write("MSR\tISR\tOSR\tISF\tMSF\tOSF\tSF\tSR\n")
    outfile.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" %
                  (int(MSR_total), int(ISR_total), int(OSR_total),
                   int(ISF_total), int(MSF_total),
                   int(OSF_total), int(SF_total), int(SR_total)))

    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



"""split mapped reads in a bam files into shorter segments.
"""


import pysam
import sys
import tempfile
import copy
import cgatcore.iotools as iotools
import cgatcore.experiment as E
from cgat.BamTools.bamtools import bam2bam_split_reads


def main(argv=sys.argv):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-i", "--input-bam", dest="input_bam_file", type="string",
        help="input bam file [%default]")

    parser.add_option(
        "-o", "--output-bam", dest="output_bam_file", type="string",
        help="input bam file [%default].")

    parser.add_option(
        "-r", "--max-read-length", dest="max_read_length", type="int",
        help="maximum read length [%default].")

    parser.add_option(
        "-m", "--output-mode", dest="output_mode", type="choice",
        choices=["buffered", "direct"],
        help="output mode for files. 'buffered' will output reads in correct "
        "sort order, 'direct' will require the output BAM file to be sorted separately."
        "[%default].")

    parser.add_option(
        "--region", dest="region", type="string",
        help="genomic region, only split in BAM file within this region [%default].")

    parser.set_defaults(
        input_bam_file="-",
        output_bam_file="-",
        max_read_length=100,
        default_quality_score=10,
        region=None,
        output_mode="buffered",
    )

    (options, args) = E.start(parser, argv)

    pysam_in = pysam.Samfile(options.input_bam_file, "rb")
    pysam_out = pysam.Samfile(options.output_bam_file, "wb", template=pysam_in)

    max_read_length = options.max_read_length

    bam2bam_split_reads(pysam_in, pysam_out,
                        default_quality_score=options.default_quality_score,
                        max_read_length=options.max_read_length,
                        output_mode=options.output_mode)

    E.stop()

if __name__ == "__main__":
    sys.exit(main())



'''
transfac2transfac.py - filter transfac motif files
====================================================

:Tags: Python

Purpose
-------

Filter a transfac motif file.

Usage
-----

Example::

   python cgat_script_template.py

Type::

   python cgat_script_template.py --help

for command line help.

Command line options
--------------------

'''


import sys
import re
import cgatcore.experiment as E


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-f", "--filter-prefix", dest="filter_prefix", default=None,
        help="ID prefix to filter on, eg. V for vertebrates")

    parser.add_option(
        "-p", "--pattern-identifier", dest="filter_pattern", default=None,
        help="ID pattern to filter (filter is case insensitive) eg. pax6. "
        "Multiple patterns should be specified as a comma separated list")

    (options, args) = E.start(parser)

    if options.filter_pattern:
        patterns = [x.strip() for x in options.filter_pattern.split(",")]
        E.info("Supplied patterns %s" % ", ".join(patterns))
    else:
        patterns = False

    filtered_motifs = []
    n = 0

    inmotif, tid, filter_emit, pattern_emit = False, False, False, False

    for line in options.stdin:

        # pick up motif start and ends.
        if line.startswith("AC") and inmotif is False:
            # print "in align"
            inmotif = True
            motif = line
            continue
        elif line.startswith("ID") and inmotif is True:
            # print line
            tid = line.split("  ")[1]
            motif += line
            continue

        elif line.startswith("//") and inmotif is True:

            motif += line

            if tid is False:
                raise ValueError("matrix ID not determined")

            if options.filter_prefix:
                if tid.startswith(options.filter_prefix):
                    filter_emit = True
            else:
                filter_emit = True

            if patterns is not False:
                for pat in patterns:
                    match = re.search(pat, tid, re.IGNORECASE)
                    if match is not None:
                        pattern_emit = True
                        break
            else:
                pattern_emit = True

            if filter_emit is True and pattern_emit is True:
                filtered_motifs.append(motif)
                n += 1

            inmotif, tid, filter_emit, pattern_emit = (
                False, False, False, False)
            continue

        elif inmotif is True:
            motif += line

        elif inmotif is False:
            continue

        else:
            raise ValueError("unknown parsing state")

    options.stdout.write("".join(filtered_motifs))

    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



"""convert fasta to VCF
==========================

Output a file in VCF format with variants according
to a fasta file.


"""

import sys
import random
import cgatcore.experiment as E
import pysam


def main(argv=None):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-s", "--sample-size", dest="sample_size", type="float",
        help="sample size. If less than 0, take a proportion of the chromosome size. "
        "If greater than 0, take a fixed number of variants [%default]")

    parser.set_defaults(
        input_filename_fasta=None,
        sample_size=0.001,
        sample_name="NA12878"
    )

    (options, args) = E.start(parser,
                              argv=argv,
                              add_output_options=True)

    if len(args) > 0:
        options.input_filename_fasta = args[0]

    if options.input_filename_fasta == "-":
        options.input_filename_fasta = options.stdin

    outf = options.stdout
    outf.write("##fileformat=VCFv4.1\n")
    outf.write("##FORMAT=<ID=GT,Number=1,Type=String,Description=\"Genotype\">\n")
    outf.write("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t{}\n".format(options.sample_name))

    with pysam.FastxFile(options.input_filename_fasta) as inf:
        for record in inf:
            contig = record.name
            sequence = record.sequence
            if options.sample_size < 1.0:
                nsamples = int(float(len(sequence)) * options.sample_size)
            else:
                nsamples = int(options.sample_size)
            E.info("generating {} sampled variants for contig {}".format(nsamples, contig))
            sampled_positions = set()
            missing_nsamples = nsamples
            while len(sampled_positions) < nsamples:
                raw_positions = random.sample(list(range(len(sequence))), nsamples - len(sampled_positions))
                filtered_positions = [x for x in raw_positions if sequence[x] != "N"]
                sampled_positions.update(filtered_positions)
                E.debug("sample update: total={}, raw={}, filtered={}".format(
                        len(sampled_positions),
                        len(raw_positions),
                        len(filtered_positions)))

            sampled_positions = sorted(sampled_positions)

            for position in sampled_positions:
                base = sequence[position]
                outf.write("{}\t{}\t.\t{}\t{}\t.\t.\t.\tGT\t0/0\n".format(
                        contig, position + 1, base, base))

    E.stop()


if __name__ == "__main__":
    sys.exit(main())



"""plot-variant-stats
=====================

Implemeted methods
------------------

plot-mutational-profile
+++++++++++++++++++++++

Plot mutational profile from output of vcf-stats.

"""

import sys
import pandas
import numpy
import re
import matplotlib.pyplot as plt
import cgatcore.experiment as E
from cgat.Plots.VariantPlots import MutationProfileBarPlot, DepthProfilePlot, \
    ManhattanPlot


def plot_mutation_profile_bar_plot(dataframe, section, map_key2label={}, **kwargs):

    for key, dataframe in dataframe.groupby(by="sample"):
        if key == "unique":
            continue

        if dataframe.empty:
            E.warn("no data for {}".format(key))
            continue

        ax = MutationProfileBarPlot()(dataframe)

        label = map_key2label.get(key, key)
        plt.savefig(E.get_output_file("-".join((section, label))))
        plt.close()


def plot_depth_profile_plot(dataframe, section, map_key2label={}, **kwargs):

    ax = DepthProfilePlot()(dataframe, map_sample2label={})
    plt.savefig(E.get_output_file(section))
    plt.close()


def plot_manhattan_plot(dataframe,
                        section,
                        filename_fasta,
                        map_key2label={},
                        **kwargs):

    plotter = ManhattanPlot(genome_size_file=filename_fasta)
    ax = plotter(dataframe, **kwargs)
    plt.savefig(E.get_output_file(section))
    plt.close()


def compute_log_depth_ratio(dataframe, min_depth=10):

    table = dataframe.set_index(["CHROM", "POS"])

    if len(table.columns) != 2:
        raise NotImplementedError("expected 2 columns in table {}".format(fn))

    normed_table = table / table.sum()
    # assumption: column order is normal, tumour
    normal, tumour = normed_table.columns
    logratios = numpy.log2(normed_table[tumour] / normed_table[normal])

    normed_table["l2fold_DP"] = logratios
    result = normed_table.drop([normal, tumour], axis=1)

    result = result[table[normal].gt(min_depth) |
                    table[tumour].gt(min_depth)]
    return result.reset_index()


def main(argv=None):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-m", "--method", dest="method", type="choice",
        choices=["mutation-profile-bar-plot",
                 "depth-profile-line-plot",
                 "manhattan-plot"],
        help="methods to apply [%default]")

    parser.add_option(
        "-t", "--transformation", dest="transformations", type="choice",
        action="append",
        choices=["log-depth-ratio"],
        help="dataframe transformation options [%default]")

    parser.add_option(
        "-r", "--regex-filename", dest="regex_filename", type="string",
        help="[%default]")

    parser.add_option(
        "-f", "--reference-fasta-file", dest="reference_fasta_file",
        help="reference genomic sequence in fasta format. "
        "[%default]")

    parser.add_option(
        "--input-file-format", dest="input_file_format", type="choice",
        choices=("tsv", "bcftools-query"),
        help="input file format "
        "[%default]")

    parser.add_option(
        "--plot-options", dest="plot_options", type="string",
        help="plot options to pass through to the plotter. The string is "
        "eval'ed, for example: --plot-options='window_size=3, ylabel=\"12\"' "
        "[%default]")

    parser.set_defaults(
        method=None,
        reference_fasta=None,
        input_file_format="tsv",
        plot_options=None,
        transformations=[],
    )

    (options, args) = E.start(parser,
                              argv=argv,
                              add_output_options=True)

    filenames = args

    if len(filenames) == 0:
        E.info("reading from stdin")
        filenames = [options.stdin]

    if options.plot_options is not None:
        plot_options = eval("dict({})".format(options.plot_options))
    else:
        plot_options = {}

    for index, filename in enumerate(filenames):

        E.info("working on {}".format(filename))

        try:
            if options.input_file_format == "bcftools-query":
                # for bctools query, header starts with "#".

                dataframe = pandas.read_csv(filename,
                                            sep="\t",
                                            skip_blank_lines=False,
                                            header=0,
                                            dtype={"CHROM": str})
                # names are of format [1]sample1:DP, extract sample1
                dataframe.columns = (
                    [re.search("\[\d+\]([^:]+)", x).groups()[0]
                     for x in dataframe.columns])
            else:
                dataframe = pandas.read_csv(filename, sep="\t",
                                            dtype={"CHROM": str})
        except pandas.io.common.EmptyDataError:
            E.warn("no data in {}, skipped".format(filename))
            continue

        E.info("read data from {}".format(filename))

        if options.regex_filename:
            section = re.search(options.regex_filename, filename).groups()[0]
        else:
            section = "{}".format(index + 1)

        for method in options.transformations:
            if method == "log-depth-ratio":
                dataframe = compute_log_depth_ratio(dataframe)

        if dataframe.empty:
            E.warn("dataframe from {} is empty - skipped".format(filename))
            continue

        if options.method == "mutation-profile-bar-plot":
            plot_mutation_profile_bar_plot(dataframe, section, **plot_options)

        elif options.method == "depth-profile-line-plot":
            plot_depth_profile_plot(dataframe, section, **plot_options)

        elif options.method == "manhattan-plot":
            plot_manhattan_plot(dataframe,
                                section,
                                filename_fasta=options.reference_fasta_file,
                                **plot_options)

    E.stop()


if __name__ == "__main__":
    sys.exit(main())
